@@ -33,7 +33,7 @@ hstring = str  # Declare where we expect a hex encoded string of length 8
 InputModel = Union[Model, QuerySet, Tuple[str, str]]  # Input models to these functions can take a number of forms
 
 
-def inputmodel_parse(inputmodel: InputModel) -> Tuple[QuerySet, Model]:
+def inputmodel_parse(inputthing: InputModel) -> Tuple[QuerySet, Model]:
     """
     Parameters
     ----------
@@ -44,15 +44,15 @@ def inputmodel_parse(inputmodel: InputModel) -> Tuple[QuerySet, Model]:
     Queryset and model from an input
     """
     # Permit either a whole Model or a given Query to be used
-    if hasattr(inputmodel, "objects"):
-        return inputmodel.objects.all(), inputmodel  # type: ignore
-    elif hasattr(inputmodel, "model"):
-        return inputmodel, inputmodel.model  # type: ignore
-    elif isinstance(inputmodel, tuple):
-        _m = apps.get_model(inputmodel[0], inputmodel[1])  # type: ignore
+    if hasattr(inputthing, "objects"):
+        return inputthing.objects.all(), inputthing  # type: ignore
+    elif hasattr(inputthing, "model"):
+        return inputthing, inputthing.model  # type: ignore
+    elif isinstance(inputthing, tuple):
+        _m = apps.get_model(inputthing[0], inputthing[1])  # type: ignore
         query = _m.objects.all()  # type: QuerySet
         return query, _m  # type: ignore
-    raise TypeError(f"Could not determine the model or queryset from {inputmodel}")
+    raise TypeError(f"Could not determine the model or queryset from {inputthing}")
 
 
 def query_to_key(query: QuerySet, suffix: str = "") -> str:
@@ -306,18 +306,30 @@ class TimeStampedFingerprint(Fingerprinting):
     Where a class or query with an "auto_now" field is present
     we can use that field instead of calculating the md5sum of all rows
     """
+
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
+        if "fingerprint_expiry" not in kwargs:
+            # The default expiry of 30s is probably too much for
+            # the very low cost of a single last-modified
+            # field, here we reduce it to 100ms
+            self.fingerprint_expiry = 0.1
+
+        # Set different hash keys for Timestamped vs hash key queries
+        # Mostly of interest in testing
+        self.cache_key = f"{self.cache_key}_ts"
+        self.time_cache_key = f"{self.time_cache_key}_ts"
+
         # Use the specified column value, if provided;
         # otherwise search for a column with an 'auto_now' field
 
         self.timestamp_column = None
-        if 'timestamp_column' in kwargs:
-            self.timestamp_column = kwargs.pop('timestamp_column')
+        if "timestamp_column" in kwargs:
+            self.timestamp_column = kwargs.pop("timestamp_column")
         else:
             for field in self.model._meta.fields:
-                if hasattr(field, 'auto_now') and field.auto_now is True:
+                if hasattr(field, "auto_now") and field.auto_now is True:
                     self.timestamp_column = field.name
                     break
         if not self.timestamp_column:
@@ -325,6 +337,10 @@ class TimeStampedFingerprint(Fingerprinting):
         logger.debug("using %s as timestamp column", self.timestamp_column)
 
     def query_fingerprint(self):
+        """
+        Returns the last updated time of the table or query rather than the
+        hash of all query rows
+        """
         ordered_query = self.query.order_by(self.timestamp_column)
         last_updated = ordered_query.last()
         last_timestamp = getattr(last_updated, self.timestamp_column)  # type: Union[datetime.date, datetime.datetime]
@@ -332,6 +348,65 @@ class TimeStampedFingerprint(Fingerprinting):
         return last_timestamp.isoformat()
 
 
+class ModelTimeStampedFingerprint(TimeStampedFingerprint):
+    """
+    This class filters the "has_changed" return to check the last updated
+    time for the query's whole model before running the fingerprint query
+    which may have a slower result
+    In many cases this should return faster than timestamp query over a few rows
+    as it avoids the filtering steps; in the worst case it adds one additional
+    but very fast query so it should probably be used as the default where
+    a model has a timestamped column unless you have a huge table and
+    a simple query
+    """
+
+    def __init__(self, *args, **kwargs):
+        super().__init(*args, **kwargs)
+        self.table_cache_key = f"{self.cache_key}_table"
+        self.table_time_cache_key = f"{self.time_cache_key}_table"
+
+    def _get_table_fingerprint(self):
+        ordered_query = self.model.order_by(self.timestamp_column)
+        last_updated = ordered_query.last()
+        last_timestamp = getattr(last_updated, self.timestamp_column)  # type: Union[datetime.date, datetime.datetime]
+        # Expect a `isoformat` on this field
+        return last_timestamp.isoformat()
+
+    @property
+    def _cached_table_fingerprint(self):
+        """
+        Return the cached hash of the query's fingerprinting result
+        """
+        return self.cache.get(self.table_cache_key)
+
+    @_cached_table_fingerprint.setter
+    def _cached_table_fingerprint(self, value):
+        """
+        Sets the cached key and also the "sentinel" value of the last time the
+        key was changed for time based validity checks
+        """
+        self.cache.set(self.table_time_cache_key, datetime.datetime.now().timestamp())
+        self.cache.set(self.table_cache_key, value)
+
+    @_cached_table_fingerprint.deleter
+    def _cached_table_fingerprint(self):
+        self.cache.delete(self.table_time_cache_key)
+        self.cache.delete(self.table_cache_key)
+
+    def update_required(self, force_check=False) -> bool:
+        """
+        Shortcut if the table has not changed since last checked
+        """
+        table_fp = self._get_table_fingerprint()
+        if table_fp == self._cached_table_fingerprint:
+            self._cached_table_fingerprint = table_fp
+            logger.debug("Table not updated")
+            return False
+        self._cached_table_fingerprint = table_fp
+        logger.debug("Table may have changed. Now checking if query has changed")
+        return super().update_required(force_check=force_check)
+
+
 class CachedQuerySet:
     """
     Cache the serialized results of a query, using a 'Fingerprinting'
